package gov.grants.apply.agency;

import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.AgencyIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ConfirmApplicationDeliveryRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.ConfirmApplicationDeliveryResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.EncodingTypeEnum;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListRequestApplicationFilterFilter;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationListResponse;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationRequest;
import gov.grants.apply.WebServices.AgencyIntegrationServices_V1_0.GetApplicationZipRequest;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.Security;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;
import org.w3c.dom.Node;

/**
 * Junit Test Case for invoking the Grants.gov Agency Web Services. A full
 * implemenation of GetApplication, GetApplicationList, GetApplicationZip and
 * ConfirmApplicationDelievery is provided. A full implementation of
 * GetApplicationZip is provided that writtes the zip files to a configurable
 * directory.
 * 
 * Support for both DIME and MIME attachment encoding specifications are enabled
 * in this implemenation.
 * 
 * @author Brian Husted
 * @July 05, 2005
 */
public class WSTestClient extends TestCase {

    private static final String GOV_GRANTS_AGENCY_ENDPOINT = "gov.grants.agency.endpoint";
   private static Properties props = null;

    public static final String PROP_FILE = "/resources/application.properties";

    static final Logger log = Logger.getLogger(WSTestClient.class.getName());

    private static AgencyIntegrationSoapBindingStub stub = null;

    public static final String LOG_4J_CONFIG_FILE = "log4j.xml";

    public static final String RESOURCES_LOG_4J_CONFIG_FILE = "/resources/"
            + LOG_4J_CONFIG_FILE;

    //Properties from application.properties in /resources /////////////
    public static final String DIR_TO_WRITE_APPS = "DIR_TO_WRITE_APPS";

    public static final String SOAP_HOST = "SOAP_HOST";

    public static final String SOAP_PORT = "SOAP_PORT";

    public static final String TEST_MODE = "TEST_MODE";

    public static final String ENCODING_TYPE = "ENCODING_TYPE";

    public static final String DIME_ENCODING = "DIME";

    ////////////////////////////////////////////////////////////////////

    static final int BUFFER = 4096;

    static {
        init();
    }

    /**
     * Constructor
     * 
     * @param str
     */
    /**
     * @param str
     */
    public WSTestClient(String str) {
        super(str);
    }

    private static void init() {
        try {
            DOMConfigurator.configure(WSTestClient.class
                    .getResource(RESOURCES_LOG_4J_CONFIG_FILE));
            log
                    .debug("log4J Config XML file loaded from /resources on classpath");

            System.setProperty("javax.net.ssl.keyStore",
                    getProperty("javax.net.ssl.keyStore"));
            System.setProperty("javax.net.ssl.keyStorePassword",
                    getProperty("javax.net.ssl.keyStorePassword"));
            System.setProperty("javax.net.ssl.trustStore",
                    getProperty("javax.net.ssl.trustStore"));
            System.setProperty("javax.net.ssl.trustStorePassword",
                    getProperty("javax.net.ssl.trustStorePassword"));
            Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
            String wsURL = getProperty(GOV_GRANTS_AGENCY_ENDPOINT);
            
            if (wsURL == null) {
               wsURL = getProperty(SOAP_HOST) + getProperty(SOAP_PORT);
            }
            
            if (wsURL == null) {
                throw new RuntimeException(
                        "SOAP HOST URL is not defined in props");
            }
            java.net.URL endpoint = new URL(wsURL);
            stub = new AgencyIntegrationSoapBindingStub(endpoint, null);

        } catch (Exception ex) {
            log.fatal("Unable to initialize", ex);
            throw new RuntimeException(ex.getMessage());
        }

    }

    /**
     * Invokes the Grants.gov GetApplicationList Web Service and logs the number
     * of applications in the list.
     * 
     * @throws Exception
     */
    public static void testGetApplicationList() throws Exception {
        ApplicationInformationType[] list = getApplicationList();
        if (list == null)
        {
           log.debug("GetApplicationList returned null");
        }
        else
        {
           log.debug("Success! Received " + list.length + " applications in list");
        }
    }

    /**
     * Invokes the Grants.gov GetApplicationZip web service. First application
     * returned from the Grants.gov GetApplicationList web service is used. The
     * zip is then written to disk.
     * 
     * @throws Exception
     */
    public static void testGetApplicationZip() throws Exception {
        String grants_gov_tracking_number = getFirstTrackingIdFromList();
        if (grants_gov_tracking_number != null) {
            AttachmentPart zip = getApplicationZip(grants_gov_tracking_number);
            writeZipToFileSystem(zip, grants_gov_tracking_number);
            log.debug("Success! Wrote Application Zip: "
                    + getProperty(DIR_TO_WRITE_APPS) + "/"
                    + grants_gov_tracking_number + ".zip");
        }
    }

    /**
     * Invokes the GetApplication Grants.gov web service. First application
     * returned from the Grants.gov GetApplicationList web service is used. The
     * XML and attachments are returned.
     * 
     * @throws Exception
     */
    public static void testGetApplication() throws Exception {

        String trackingNumber = getFirstTrackingIdFromList();

        if (trackingNumber != null) {
            Node node = getApplication(trackingNumber);
            log.debug("Retrieved Application " + trackingNumber);
            log.debug(node);

        }
    }

    /**
     * Invokes the ConfirmApplicationDelivery Grants.gov web service.
     * 
     * @throws Exception
     */
    public static void testConfirmApplicationDelivery() throws Exception {
        String grants_gov_tracking_number = getFirstTrackingIdFromList();
        if (grants_gov_tracking_number != null) {

            confirmApplication(grants_gov_tracking_number);
            log.debug("Success! confirmed tracking number "
                    + grants_gov_tracking_number);
        }
    }

    protected static String getFirstTrackingIdFromList() throws Exception {

        ApplicationInformationType[] list = getApplicationList();
        if (list == null || list.length <= 0) {

            String msg = "Unable to retrieve a zip since "
                    + "getApplicationList returned ZERO apps";
            log.debug(msg);
            return null;

        } else {
            return list[0].getGrants_govTrackingNumber();

        }
    }

    /**
     * Retrieves all the applications by calling GetApplicationList. For each
     * application, the GetApplicationZip web service is invoked and the zip is
     * written to disk. If everything is successful, the
     * ConfirmApplicationDelievery web service is called
     * 
     * @throws Exception
     */
    public static void processApplications() throws Exception {

        ApplicationInformationType[] apps = getApplicationList();

        if (apps != null) {
            log.debug("receiving " + apps.length + " applications");
            for (int i = 0; i < apps.length; i++) {
                String application_id = apps[i].getGrants_govTrackingNumber();
                processApplication(application_id);
                log.debug("Successfully processed Application!");

            }
        } else {
            log.debug("Received zero applications");
        }

    }

    protected static ApplicationInformationType[] getApplicationList()
            throws Exception {

        GetApplicationListRequestApplicationFilter[] filter = new GetApplicationListRequestApplicationFilter[1];
        GetApplicationListRequestApplicationFilterFilter af_filter = GetApplicationListRequestApplicationFilterFilter
                .fromString("Status");

        filter[0] = new GetApplicationListRequestApplicationFilter();
        filter[0].setFilter(af_filter);
        if (Boolean.getBoolean(TEST_MODE)) {
            log.debug("test mode");
            filter[0].setFilterValue("Received by Agency");
        } else {
            log.debug("production mode");
            filter[0].setFilterValue("Validated");
        }

        GetApplicationListResponse response = stub.getApplicationList(filter);

        return response.getApplicationInformation();
    }

    protected static void processApplication(String application_id)
            throws Exception {
        AttachmentPart zip = getApplicationZip(application_id);
        writeZipToFileSystem(zip, application_id);
        confirmApplication(application_id);

    }

    protected static AttachmentPart getApplicationZip(String applicationId)
            throws Exception {

        String trackingNum = applicationId;
        GetApplicationZipRequest request = new GetApplicationZipRequest();
        //Set Encoding type to receive attachments
        //DIME or MIME
        request.setEncodingType(getAttachmentEncodingType());
        request.setGrants_govTrackingNumber(trackingNum);
        log.debug("retrieving application zip for tracking ID: "
                + applicationId);
        Object ret = stub.getApplicationZip(request);
        log.debug("got application zip for tracking ID: " + applicationId);
        return (AttachmentPart) stub.getAttachments()[0];

    }

    protected static Node getApplication(String applicationId) throws Exception {

        String trackingNum = applicationId;
        GetApplicationRequest request = new GetApplicationRequest();
        //Set Encoding type to receive attachments
        //DIME or MIME
        request.setEncodingType(getAttachmentEncodingType());
        request.setGrants_govTrackingNumber(trackingNum);
        System.out.println("retrieving application info for tracking ID: "
                + applicationId);
        Node n = (Node) stub.getApplication(request);
        //Getting the attachments
        Object[] att = stub.getAttachments();
        if (att.length > 0) {
            log.debug(att.length + " attachments ");
            for (int i = 0; i < att.length; i++) {
                log.debug("writing attachment to file system in ");

                writeAttachmentToFileSystem((AttachmentPart) att[i],
                        applicationId);
            }
        } else {
            log.debug("No Attachments available");
        }
        return n;
    }

    protected static EncodingTypeEnum getAttachmentEncodingType()
            throws Exception {
        if (DIME_ENCODING.equalsIgnoreCase(getProperty(ENCODING_TYPE))) {
            return EncodingTypeEnum.DIME;
        }

        return EncodingTypeEnum.MIME;
    }

    private static void writeZipToFileSystem(AttachmentPart part,
            String applicationId) throws Exception {
        InputStream is = (InputStream) part.getContent();
        ZipInputStream in = null;
        ZipOutputStream out = null;

        File zip = new File(getProperty(DIR_TO_WRITE_APPS), applicationId
                + ".zip");
        try {
            in = new ZipInputStream(is);
            out = new ZipOutputStream(new BufferedOutputStream(
                    new FileOutputStream(zip)));

            ZipEntry ze = in.getNextEntry();

            while (ze != null) {

                log.debug("writing blob for: " + ze.getName());
                addZipEntry(in, out, ze);

                in.closeEntry();
                ze = in.getNextEntry();
            }
        } finally {
            try {
                in.close();
            } catch (Exception ex) {
            }
            try {
                out.flush();
            } catch (Exception ex) {
            }
            try {
                out.close();
            } catch (Exception ex) {
            }
        }
    }

    private static void writeAttachmentToFileSystem(AttachmentPart part,
            String applicationId) throws Exception {
        InputStream is = (InputStream) part.getContent();

        BufferedOutputStream out = null;

        File attachment = new File(getProperty(DIR_TO_WRITE_APPS),
                applicationId + "_" + System.currentTimeMillis());
        try {

            out = new BufferedOutputStream(new FileOutputStream(attachment));

            byte[] bytes = new byte[4096];
            int ctr = 0;
            while ((ctr = is.read(bytes)) > 0) {
                out.write(bytes, 0, ctr);
            }

            log.debug("Finished writing " + attachment.getAbsolutePath());

        } finally {

            try {
                out.flush();
            } catch (Exception ex) {
            }
            try {
                out.close();
            } catch (Exception ex) {
            }
        }
    }

    private static void addZipEntry(ZipInputStream in, ZipOutputStream out,
            ZipEntry ze) throws IOException {

        byte data[] = new byte[BUFFER];
        try {

            out.putNextEntry(ze);
            int count;
            while ((count = in.read(data, 0, BUFFER)) >= 0) {
                out.write(data, 0, count);
            }
            out.flush();

        } finally {
            try {
                out.closeEntry();
            } catch (Exception ex) {
            }
        }
    }

    protected static void confirmApplication(String application_id)
            throws Exception {
        log.debug("confirming delivery: " + application_id);
        ConfirmApplicationDeliveryRequest request = new ConfirmApplicationDeliveryRequest();
        String trackingNum = application_id;
        request.setGrants_govTrackingNumber(trackingNum);
        ConfirmApplicationDeliveryResponse resp = stub
                .confirmApplicationDelivery(request);
        log.debug("response message: " + resp.getResponseMessage().getValue());
    }

    /**
     * Get a property value from the soap server properties file.
     * 
     * @param key
     *            An key value
     * @return The property value or null if no property exists
     * @throws IOException
     */
    private static String getProperty(String key) throws IOException {
        if (props == null) {
            synchronized (WSTestClient.class) {
                if (props == null) {
                    props = loadProperties();
                }
            }

        }

        return props.getProperty(key).trim();

    }

    /**
     * Load Properties
     * 
     * @return The Properties
     * @throws IOException
     */
    private static Properties loadProperties() throws IOException {
        Properties props = new Properties();

        InputStream stream = null;
        try {
            stream = WSTestClient.class.getResourceAsStream(PROP_FILE);
            log.debug("application.properties file loaded from " + PROP_FILE
                    + " on classpath");

            props.load(stream);

        } finally {
            try {
                stream.close();
            } catch (Exception ex) {
            }
        }
        return props;
    }

    public static void main(String[] args)
       throws Exception
    {
        TestSuite suite = new TestSuite(WSTestClient.class);
        TestRunner.run(suite);
    }

}