/*
 * Created on Jun 1, 2005
 *
 * Unpublished Work, Copyright, 2004 Northrop Grumman
 */
package gov.grants.apply.ws;

import gov.grants.apply.util.PropertiesLoader;
import gov.grants.apply.ws.production.ProductionModeFactory;
import gov.grants.apply.ws.standAloneMode.StandAloneModeFactory;

import org.apache.log4j.Logger;

/**
 * @author Grants.gov WebServices Team
 *
 * Abstract factory deciding what type of processor should be used thru the life of the
 * application.
 */
public abstract class ProcessorFactory
{
   private static final String PRODUCTION_MODE_KEY = "PRODUCTION_MODE";
   private static final String PROCESSOR_CLASS_KEY = "PROCESSOR_CLASS";
   private final static Logger LOG = Logger.getLogger(ProcessorFactory.class);
   private final static String APPLICATION_PROP_FILE = "application.properties";
   private final static ProcessorFactory _factory;

   static
   {
      ProcessorFactory factory = null;
      String className = PropertiesLoader.getValue(APPLICATION_PROP_FILE, PROCESSOR_CLASS_KEY);
      boolean prodMode = PropertiesLoader.getBooleanValue(APPLICATION_PROP_FILE, PRODUCTION_MODE_KEY);
      
      if (!prodMode && className == null)
      {
         factory = new StandAloneModeFactory();
      }
      else if (!prodMode && className != null)
      {
         LOG.debug("Production Mode configuration parameter was not set or was set to false.");
         if (className != null)
         {
            try
            {
               Class factoryClass = Class.forName(className);
               if (ProcessorFactory.class.isAssignableFrom(factoryClass))
               {
                  factory = (ProcessorFactory) factoryClass.newInstance();
               }
               else
               {
                  LOG.error("Custom process factory MUST be of type " + ProcessorFactory.class.getName());
               }
            }
            catch (InstantiationException e)
            {
               LOG.error("Could not instantiate custom processor factory class.", e);
            }
            catch (IllegalAccessException e)
            {
               LOG.error("Could not instantiate custom processor factory class.", e);
            }
            catch (ClassNotFoundException e)
            {
               LOG.error("Could not instantiate custom processor factory class.", e);
            }
         }
         else
         {
            LOG.debug("Processor class name was not specified in configuration");
         }
      }

      if (factory == null)
      {
         // if everything failed above, then the system defaults to production mode (i.e. this handles all exceptions above)
         LOG.info("Since PRODUCTION_MODE was not \"false\" and the PROCESSOR_CLASS name was not provided or could not be instantiated, the production mode processors will be used.");
         _factory = new ProductionModeFactory();
      }
      else
      {
         _factory = factory;
         if (LOG.isDebugEnabled())
         {
            LOG.debug("Using processor factory: " + _factory.getClass().getName());
         }
      }
   }
   
   public final static ProcessorFactory getFactory()
   {
      return _factory;
   }
   
   public final static boolean isProductionMode()
   {
      return PropertiesLoader.getBooleanValue(APPLICATION_PROP_FILE, PRODUCTION_MODE_KEY);
   }
   
   public abstract IAssignAgencyTrackingNumberProcessor createAssignAgencyTrackingNumberProcessor();
   public abstract IConfirmApplicationDeliveryProcessor createConfirmApplicationDeliveryProcessor();
   public abstract IGetApplicationListProcessor createGetApplicationListProcessor();
   public abstract IGetApplicationProcessor createGetApplicationProcessor();
   public abstract IGetApplicationZipProcessor createGetApplicationZipProcessor();
}
